'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var _ = require('lodash');
var Redis = require('ioredis');
var uid = require('uid-safe');

exports = module.exports = function (opts) {
  var defaultOptions = {
    maxAge: 1000 * 60 * 60,
    prefix: 'afcsrf',
    redis: {
      host: '127.0.0.1',
      prot: 6379,
      db: 0
    }
  };

  var tokens = require('csrf')(opts);

  var RedisStore = function () {
    function RedisStore() {
      _classCallCheck(this, RedisStore);

      this.redis = new Redis(opts.redis);
    }

    _createClass(RedisStore, [{
      key: 'set',
      value: function () {
        var _ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(csrfKey, secret, csrf) {
          var key;
          return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  key = opts.prefix + ':' + csrfKey;
                  _context.next = 3;
                  return this.redis.sadd(key, csrf + ':' + secret);

                case 3:
                  _context.next = 5;
                  return this.redis.expire(key, opts.maxAge);

                case 5:
                  return _context.abrupt('return', true);

                case 6:
                case 'end':
                  return _context.stop();
              }
            }
          }, _callee, this);
        }));

        function set(_x, _x2, _x3) {
          return _ref.apply(this, arguments);
        }

        return set;
      }()
    }, {
      key: 'verify',
      value: function () {
        var _ref2 = _asyncToGenerator(regeneratorRuntime.mark(function _callee2(csrfKey, csrf) {
          var key, set, tiket;
          return regeneratorRuntime.wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  key = opts.prefix + ':' + csrfKey;
                  _context2.next = 3;
                  return this.redis.smembers(key);

                case 3:
                  set = _context2.sent;
                  tiket = _.find(set, function (t) {
                    return t.indexOf(csrf + ':') === 0;
                  });

                  if (!tiket) {
                    _context2.next = 9;
                    break;
                  }

                  _context2.next = 8;
                  return this.redis.srem(key, tiket);

                case 8:
                  return _context2.abrupt('return', true);

                case 9:
                  return _context2.abrupt('return', false);

                case 10:
                case 'end':
                  return _context2.stop();
              }
            }
          }, _callee2, this);
        }));

        function verify(_x4, _x5) {
          return _ref2.apply(this, arguments);
        }

        return verify;
      }()
    }]);

    return RedisStore;
  }();

  opts = _.defaultsDeep(opts, defaultOptions);

  var store = new RedisStore();

  var define = function define(ctx) {
    ctx.getCsrf = _asyncToGenerator(regeneratorRuntime.mark(function _callee3() {
      var csrfKey, secret, csrf, result;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              /**
               * generate token
               */
              csrfKey = ctx.session._CSRFKEY;

              if (!csrfKey) {
                csrfKey = uid.sync(24);
                ctx.session._CSRFKEY = csrfKey;
              }
              secret = tokens.secretSync();
              csrf = tokens.create(secret);
              _context3.next = 6;
              return store.set(csrfKey, secret, csrf);

            case 6:
              result = _context3.sent;

              if (result) {
                _context3.next = 9;
                break;
              }

              throw new Error('Get csrf error!');

            case 9:
              return _context3.abrupt('return', csrf);

            case 10:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined);
    }));

    ctx.assertCSRF = ctx.assertCsrf = function () {
      var _ref4 = _asyncToGenerator(regeneratorRuntime.mark(function _callee4(body) {
        var csrfKey, csrf, result;
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                csrfKey = ctx.session._CSRFKEY;

                if (csrfKey) {
                  _context4.next = 4;
                  break;
                }

                ctx.throw(403, 'secret is missing');
                return _context4.abrupt('return', false);

              case 4:
                csrf = body && body._csrf || ctx.query && ctx.query._csrf || ctx.get('x-csrf-token') || ctx.get('x-xsrf-token') || body;

                if (csrf) {
                  _context4.next = 8;
                  break;
                }

                ctx.throw(403, 'token is missing');
                return _context4.abrupt('return', false);

              case 8:
                _context4.next = 10;
                return store.verify(csrfKey, csrf);

              case 10:
                result = _context4.sent;

                if (result) {
                  _context4.next = 14;
                  break;
                }

                ctx.throw(403, 'invalid csrf token');
                return _context4.abrupt('return', false);

              case 14:
                return _context4.abrupt('return', true);

              case 15:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, undefined);
      }));

      return function (_x6) {
        return _ref4.apply(this, arguments);
      };
    }();
  };

  var middleware = function () {
    var _ref5 = _asyncToGenerator(regeneratorRuntime.mark(function _callee5(ctx, next) {
      return regeneratorRuntime.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              if (!(ctx.method === 'GET' || ctx.method === 'HEAD' || ctx.method === 'OPTIONS')) {
                _context5.next = 4;
                break;
              }

              _context5.next = 3;
              return next();

            case 3:
              return _context5.abrupt('return');

            case 4:
              _context5.next = 6;
              return ctx.assertCSRF(ctx.request.body);

            case 6:
              if (!_context5.sent) {
                _context5.next = 9;
                break;
              }

              _context5.next = 9;
              return next();

            case 9:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, undefined);
    }));

    return function middleware(_x7, _x8) {
      return _ref5.apply(this, arguments);
    };
  }();

  return function () {
    var _ref6 = _asyncToGenerator(regeneratorRuntime.mark(function _callee6(ctx, next) {
      return regeneratorRuntime.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              define(ctx);
              _context6.next = 3;
              return middleware(ctx, next);

            case 3:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, undefined);
    }));

    return function (_x9, _x10) {
      return _ref6.apply(this, arguments);
    };
  }();
};